import * as Axios from 'axios';

declare type XomApiQueryParam =
  | '$skiptoken'
  | '$orderby'
  | '$select'
  | '$filter'
  | '$expand'
  | '$skip'
  | '$top'

declare type XomApiQueryString = Range<XomApiQueryParam, string>

declare type XomApiQueryString = XomApiQueryString | string

declare interface XomApiRequestConfig extends Axios.AxiosRequestConfig {
  interceptors?: {
    request: Axios.AxiosInterceptorManager<XomApiRequestConfig>
    response: Axios.AxiosInterceptorManager<Axios.AxiosResponse>
  }
  requestDigest?: Promise<string>
  digest?: boolean
}

declare interface XomApiResponse {
  __response: Axios.AxiosResponse
  [key: string]: any
}

declare interface XomApiClient extends Axios.AxiosInstance {
  defaults: XomApiRequestConfig
  get<T = any, R = Axios.AxiosResponse<T>>(url: string, config?: XomApiRequestConfig): Promise<R>
  delete<T = any, R = Axios.AxiosResponse<T>>(url: string, config?: XomApiRequestConfig): Promise<R>
  post<T = any, R = Axios.AxiosResponse<T>>(url: string, data?: any, config?: XomApiRequestConfig): Promise<R>
  patch<T = any, R = Axios.AxiosResponse<T>>(url: string, data?: any, config?: XomApiRequestConfig): Promise<R>
}

/**
 * Instantiate the object with the necessary information to connect to a SharePoint list through its REST API.
 */
declare class XomSharePointList {
    private _title;
    private _http;
    private _itemsType;
    constructor(listTitle: string, httpInstance: XomApiClient);
    get title(): string;
    /**
     * Returns the list fields metadata;
     */
    getFields(params?: XomApiQueryString): Promise<XomApiResponse>;
    /**
     * Returns a list of the items stored in the list.
     */
    getItems(params?: XomApiQueryString): Promise<XomApiResponse>;
    /**
     * Returns a single list item with the given ID.
     */
    findItem(itemId: number, params?: XomApiQueryString): Promise<XomApiResponse>;
    /**
     * Creates or updates a record based on an existing ID.
     */
    saveItem(data: any): Promise<XomApiResponse>;
    saveItem(itemId: number, data: any): Promise<XomApiResponse>;
    /**
     * Saves a new record in the SharePoint list.
     */
    createItem(data: any): Promise<XomApiResponse>;
    /**
     * Updates an existing record in the SharePoint list.
     */
    updateItem(item: any): Promise<XomApiResponse>;
    updateItem(itemId: number, data: any): Promise<XomApiResponse>;
    /**
     * Deletes an existing record from the SharePoint list.
     */
    deleteItem(item: any): Promise<XomApiResponse>;
    deleteItem(itemId: number): Promise<XomApiResponse>;
    /**
     * Return a list of the attached files in the list item.
     */
    getAttachments(item: any): Promise<XomApiResponse>;
    getAttachments(itemId: number): Promise<XomApiResponse>;
    /**
     * Uploads a file attachment to a list item.
     */
    addAttachment(item: any, fileName: string, fileArrayBuffer: ArrayBuffer): Promise<XomApiResponse>;
    addAttachment(item: any, fileName: string, blobInstance: Blob): Promise<XomApiResponse>;
    addAttachment(item: any, fileName: string, querySelector: string): Promise<XomApiResponse>;
    addAttachment(item: any, fileName: string, fileInputElement: HTMLInputElement): Promise<XomApiResponse>;
    addAttachment(item: any, fileName: string, fileInputContent: FileList): Promise<XomApiResponse>;
    addAttachment(item: any, fileName: string, fileInstance: File): Promise<XomApiResponse>;
    addAttachment(itemId: number, fileName: string, fileArrayBuffer: ArrayBuffer): Promise<XomApiResponse>;
    addAttachment(itemId: number, fileName: string, blobInstance: Blob): Promise<XomApiResponse>;
    addAttachment(itemId: number, fileName: string, querySelector: string): Promise<XomApiResponse>;
    addAttachment(itemId: number, fileName: string, fileInputElement: HTMLInputElement): Promise<XomApiResponse>;
    addAttachment(itemId: number, fileName: string, fileInputContent: FileList): Promise<XomApiResponse>;
    addAttachment(itemId: number, fileName: string, fileInstance: File): Promise<XomApiResponse>;
    /**
     * Renames a given file attachment.
     */
    renameAttachment(item: any, currentName: string, newName: string): Promise<XomApiResponse>;
    renameAttachment(itemId: number, currentName: string, newName: string): Promise<XomApiResponse>;
    /**
     * Removes a given file attachment from the list item.
     */
    deleteAttachment(item: any, fileName: string): Promise<XomApiResponse>;
    deleteAttachment(itemId: number, fileName: string): Promise<XomApiResponse>;
}

/**
 * Instantiate the object with the necessary information to connect to a SharePoint survey through its REST API.
 */
declare class XomSharePointSurvey {
    private _title;
    private _http;
    private _itemsType;
    constructor(surveyTitle: string, httpInstance: XomApiClient);
    get title(): string;
    /**
     * Gets fields that corresponds to the questions and their choices.
     */
    getQuestions(): Promise<any>;
    /**
     * Returns a list of the responses stored in the survey list.
     */
    getResponses(params?: XomApiQueryString): Promise<XomApiResponse>;
    /**
     * Returns a single response by its ID.
     */
    findResponse(id: number, params?: XomApiQueryString): Promise<XomApiResponse>;
    /**
     * Saves a new response in the SharePoint survey list.
     */
    submitResponse(data: any): Promise<XomApiResponse>;
    /**
     * Updates an existing response.
     */
    changeResponse(id: number, data: any): Promise<XomApiResponse>;
    /**
     * Deletes an existing response.
     */
    delete(id: number): Promise<XomApiResponse>;
}

/**
 * Instantiate the object with the necessary information to connect to a SharePoint file library through its REST API.
 */
declare class XomSharePointLibrary {
    private _address;
    private _http;
    private _filesType;
    constructor(folderAddress: string, httpInstance: XomApiClient);
    get relativeUrl(): string;
    /**
     * Returns a list of the folders within the folder.
     */
    listSubfolders(params?: XomApiQueryString): Promise<XomApiResponse>;
    /**
     * Creates a folder within this folder.
     */
    createSubfolder(folderName: string): Promise<XomApiResponse>;
    /**
     * Returns a list of the files within this folder.
     */
    listFiles(params?: XomApiQueryString): Promise<XomApiResponse>;
    /**
     * Uploads a file into the folder.
     */
    uploadFile(fileName: string, fileArrayBuffer: ArrayBuffer): Promise<XomApiResponse>;
    uploadFile(fileName: string, blobInstance: Blob): Promise<XomApiResponse>;
    uploadFile(fileName: string, querySelector: string): Promise<XomApiResponse>;
    uploadFile(fileName: string, fileInputElement: HTMLInputElement): Promise<XomApiResponse>;
    uploadFile(fileName: string, fileInputContent: FileList): Promise<XomApiResponse>;
    uploadFile(fileName: string, fileInstance: File): Promise<XomApiResponse>;
}

/**
 * Instantiate the object with the necessary information to connect to a SharePoint site through its REST API.
 */
declare class XomSharePointSite {
    private _http;
    private _currUser;
    constructor(baseSiteUrl: string);
    get http(): XomApiClient;
    get baseUrl(): string | undefined;
    /**
     * Gets the SharePoint site metadata.
     */
    getInfo(): Promise<XomApiResponse>;
    /**
     * Queries the SharePoint API to get user information. Inform nothing to get
     * current user information or pass an specific user ID.
     */
    getUserInfo(id?: number): Promise<XomApiResponse>;
    /**
     * Queries SharePoint API searching for user name.
     */
    searchUser(search: string): Promise<XomApiResponse>;
    /**
     * Returns a reference to connect to a SharePoint list.
     */
    getList(listTitle: string): XomSharePointList;
    /**
     * Creates a new SharePoint list.
     */
    createList(listTitle: string): Promise<XomApiResponse>;
    /**
     * Deletes an existing SharePoint list.
     */
    deleteList(listTitle: string): Promise<XomApiResponse>;
    /**
     * Returns a reference to connect to a SharePoint survey.
     */
    getSurvey(surveyTitle: string): XomSharePointSurvey;
    /**
     * Returns a reference to connect to a SharePoint file library.
     */
    getFolder(folderAddress: string): XomSharePointLibrary;
}

/**
 * Instantiate a XomSharePoint object to connect to a SharePoint site and,
 * therefore, exchange data with its contents (lists, libraries, permissions)
 * through SharePoint native REST API
 */
declare function xomSharePoint(baseSiteUrl: string): XomSharePointSite;

export default xomSharePoint;
